function [deltaE,Icor,Irep,c]=shiftfit(Ishift,esv,E,isshift);

%*********************************************************************
%
% shiftfit, by J.M. Daz-Cruz, May 2006; Last revised October 2007 
%
% Requires shiftcalc function
%
% Corrects the whole matrix for the potential shift of one or more 
% components (deltaE)
%
% esv (in column-wise form) is the estimation of the unit voltammograms
% at the reference potentials
% Icor, Irep are the corrected and reproduced matrices, respectively
% c is an estimation of concentration profiles (in arbitrary units)
% E is the potential vector (in a column)
% isshift is a vector in the form [1,0,0,1,....] which denotes with '1' 
% the components to be corrected for the potential shift
%
% Syntax:   [deltaE,Icor,Irep,c]=shiftfit(Ishift,esv,E,isshift);
%
%*********************************************************************

%checking dimensions and correcting matrix orientation if possible

[nexp,npot]=size(Ishift);
[npot2,ncomp]=size(esv);
[npot3,nothing]=size(E);
if npot3==1, E=E';[npot3,nothing]=size(E);end
if npot3~=npot, 
   Ishift=Ishift';[nexp,npot]=size(Ishift);
   if npot3~=npot, error('Ishift and E dimensions must agree');end
end
if npot2~=npot, 
   esv=esv';[npot2,ncomp]=size(esv);
	if npot2~=npot, error('Ishift and esv dimensions must agree');end
end


%normalized variable number of column

for i=1:npot,x(i,1)=i/npot;end

%normalization of Ishift and esv (Ishiftn and v)
maxI=max(max(Ishift));
Ishiftn=Ishift./maxI;
maxv=max(max(esv));
v=esv./maxv;
%conversion factor between delta(=f(x)) and deltaE(=f(E))
factorE=max(E)-min(E);

%estimation of delta0 (in the units of the normalized variable x)

delta0=zeros(nexp,ncomp);

 %generation of different filters to be used in Savitzky-Golay 2nd derivative
 fil63=cgolay(6,3,2);
 fil32=cgolay(3,2,2);

for i=1:nexp
   for k=1:ncomp
     if isshift(1,k)==1
            v2=v(:,k);
       % selecting the parts of the initial estimation where the peak is more than 20% of the maximum
       ipeakselect=find(v2>0.2*max(v2));
       x3=x(ipeakselect);v3=v2(ipeakselect);I3=(Ishiftn(i,ipeakselect));
       %looking for the second derivative by Savitzky-Golay if there are enough points in the range I3
       [nrI3,ncI3]=size(I3);
       if ncI3>20, dI3=(golay(fil63,I3'))';
       elseif ncI3>9, dI3=(golay(fil32,I3'))';
       elseif ncI3<=9, dI3=zeros(1,ncI3);
       end
       %looking for maxima in the experimental matrix
       [ymax,imax]=max(I3);
       %testing if there is a relative or an absolute maximum (in the last case the range is shortened)
       if dI3(imax)>0,
          iderselect=find(dI3<0);
          if isempty(iderselect)==0;
          x3=x(iderselect);v3=v2(iderselect);I3=(Ishiftn(i,iderselect));  
          [ymax,imax]=max(I3);
          else delta0(i,k)=0;
          end
       end
       %the maximun of the estimated unit voltammogram
       [yesmax,iesmax]=max(v3);
       if ymax>0.05, delta0(i,k)=x3(imax)-x3(iesmax); end
     else delta0(i,k)=0;  
     end  
   end
end
 
subplot(1,1,1); 
plot(factorE*delta0,'o');xlabel('Experiment number');ylabel('deltaE0 / V');
title('initial estimation of deltaE: press any key');pause


%estimation of c

for i=1:nexp
    for k=1:ncomp, 
        if delta0(i,k)~=0 & isshift(1,k)==1, xshift=x-delta0(i,k);
           vshift(:,k)=spline(x,v(:,k),xshift);
        else vshift(:,k)=v(:,k);
        end
    end
    c(i,:)=negtozero(Ishiftn(i,:)/vshift');
end


%fitting
options=optimset('Display','off','Diagnostics','off','MaxIter',50,'TolX',0.001,'TolFun',0.001);

for i=1:nexp
   cv=[isshift;c(i,:);v];
   [delta(i,:)]=lsqcurvefit('shiftcalc',delta0(i,:),cv,Ishiftn(i,:),0,0.3,options);
end


%conversion to deltaE

deltaE=delta*factorE;

plot(deltaE,'o');xlabel('Experiment number');ylabel('deltaE / V');
title('fitted values of deltaE: press any key');pause


%corrected matrix (Icor), 

for i=1:nexp
    for k=1:ncomp, 
        if delta(i,k)~=0, xshift=x-delta(i,k);
           vshift(:,k)=spline(x,v(:,k),xshift);
        else vshift(:,k)=v(:,k);
        end
    end
    c(i,:)=negtozero(Ishiftn(i,:)/vshift');
end

Icor=maxI*c*v';


%reproduced matrix (Irep) and lack of fit between Irep and Ishift

for i=1:nexp,Irep(i,:)=maxI*shiftcalc(delta(i,:),[isshift;c(i,:);v]); end

ss1=0;ss2=0;
for i=1:nexp
   for j=1:npot
      ss1=ss1+(Ishift(i,j)-Irep(i,j))^2;
      ss2=ss2+(Ishift(i,j))^2;
   end
end


lof=((ss1/ss2)^0.5)*100;

format short

disp(' ');disp('lack of fit (%) :  ');disp(lof)


% overall plot

plot(c);title('fitted concentration / a.u.: press any key');xlabel('number of experiment');pause

subplot(3,1,1);plot(-E,Ishift);title('experimental matrix');ylabel('current / A');
subplot(3,1,2);plot(-E,Irep);title('reproduced matrix');ylabel('current / A');
subplot(3,1,3);plot(-E,Icor);title('corrected matrix');xlabel('-E / V');ylabel('current / A');



